
// Internal file : BaseProcessing/ADISUSBZ.cpp

#include "StdAfx.h"
#include <assert.h>
#include <Windows.h>
#include "ADISUSBZ.h"

#define IOCTL_BULK_WRITE ( 0x220001 + ( 0x800 + 20 ) * 4 )
#define IOCTL_BULK_READ  ( 0x220002 + ( 0x800 + 19 ) * 4 )

ADISUSBZ::ADISUSBZ() : _handle( INVALID_HANDLE_VALUE )
{
}

ADISUSBZ::~ADISUSBZ()
{
    // If the instance have been initialized, we need to close the handle to the driver.
    if ( _handle != INVALID_HANDLE_VALUE ) {
        CloseHandle( _handle );
        _handle = INVALID_HANDLE_VALUE;
    }
}

bool ADISUSBZ::Initialize( const char * aDeviceName )
{
    assert( aDeviceName != NULL );
    assert( _handle == INVALID_HANDLE_VALUE );

    _handle = CreateFile( aDeviceName, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL );
    if ( _handle == INVALID_HANDLE_VALUE ) {
        return false;
    }

    WriteStaticBits();
    WriteLoopDelay ();

    return true;
}

short ADISUSBZ::ReadInt14( unsigned char aAddress )
{
    short lResult = ReadInt16( aAddress );

    // Sign extension from 14 bits to 16 bits
    if ((lResult & 0x2000) == 0x2000) lResult |= 0xc000;
    else							  lResult &= 0x3fff;

    return lResult;
}

// Private
/////////////////////////////////////////////////////////////////////////////

short ADISUSBZ::ReadInt16( unsigned char aAddress )
{
	unsigned char lBuffer[7] = {
		0x0d, // Polarity   = POS
		0x01, // Channel    = 1
		2,    // Byte count = 2
		aAddress,
		0,
		aAddress,
		0,
	};

    Tx( lBuffer, 7 );
    Rx( lBuffer, 4 );

    // Reverse byte order
    return (lBuffer[2]<<8) | (lBuffer[3]<<0 );
}

void ADISUSBZ::WriteLoopDelay()
{
	unsigned char lBuffer[2] = {
		0x0b,
		0,
	};

    Tx (lBuffer, 2);
}

void ADISUSBZ::WriteStaticBits()
{
	unsigned char lBuffer[2] = {
		0x0c,
		0x71,
	};

    Tx (lBuffer, 2);
}

void ADISUSBZ::Rx( void * aOut, unsigned char aOutSize )
{
    assert (aOut != NULL);
    assert (aOutSize > 0);

    unsigned int lBTCT = 1; // Bulk Transfer Control Type

    DeviceControl( IOCTL_BULK_WRITE, &lBTCT, sizeof(lBTCT), aOut, aOutSize );
}

void ADISUSBZ::Tx( const void *aIn, unsigned char aInSize )
{
    assert (aIn != NULL);
    assert (aInSize > 0);

    unsigned int lBTCT = 0; // Bulk Transfer Control Type

    DeviceControl( IOCTL_BULK_WRITE, &lBTCT, sizeof(lBTCT), const_cast< void * >( aIn ), aInSize );
}

void ADISUSBZ::DeviceControl( unsigned int aIoCtl, const void * aIn, unsigned int aInSize, void * aOut, unsigned int aOutSize )
{
    assert( _handle != INVALID_HANDLE_VALUE );

    DWORD lInfo;

    BOOL lBool = DeviceIoControl( _handle, aIoCtl, const_cast<void *>(aIn), aInSize, aOut, aOutSize, &lInfo, NULL);

	assert( lBool != FALSE    );
    assert( lInfo == aOutSize );
}
