#include "StdAfx.h"
#include "MatrixAlgebra.h"
#include "Quaternion.h"
#include "mm.h"

static dMatrix x(4,1);	// EKF  , ʹϾ   ̴.
static dMatrix P(4,4);	//   л 

double average_gyro[3] = {0, 0, 0, };	// ӵ   Ѵ. ȭ鿡 ǥϱ   .
double adjust_rpy[3] = {0, 0, 0, };		// ӵ  ߷ κ  roll, pitch, yaw . ⼭  .

void InitARS()
{
	// EKF  Ϳ л  ʱȭ Ѵ.
	x.null();		// º(ʹϾ) 0,0,0,1  ʱȭ
	x(3,0) = 1;
	P.unit();		// л  ķ ʱȭ
}

void GyroAverage (double average_gyro[3], double gyro[3], const int max_count, const double tolerance)
{
	//   ִ ӵ(gyroscope)   0   ʴ´.
	// κ Ư  ̾ Ǿִµ,   0 ϱ Ͽ
	//     ӵ   Ѵ.

	static int count = 0;

	double d0 = gyro[0] - average_gyro[0];
	double d1 = gyro[1] - average_gyro[1];
	double d2 = gyro[2] - average_gyro[2];

	if (!count || ( 
		-tolerance < d0 && d0 < tolerance && 
		-tolerance < d1 && d1 < tolerance && 
		-tolerance < d2 && d2 < tolerance )) {

		average_gyro[0]  = (average_gyro[0]*count + gyro[0])/(count+1);
		average_gyro[1]  = (average_gyro[1]*count + gyro[1])/(count+1);
		average_gyro[2]  = (average_gyro[2]*count + gyro[2])/(count+1);
	}
	if (count < max_count) ++count;
}

dMatrix GetStateTransitionMatrix(double wx, double wy, double wz, double dt)
{
	dMatrix A(4,4);

	wx *= 0.5*dt;
	wy *= 0.5*dt;
	wz *= 0.5*dt;

	A(0,0) = 1;		A(0,1) = wz;	A(0,2) = -wy;	A(0,3) = wx;
	A(1,0) = -wz;	A(1,1) = 1;		A(1,2) = wx;	A(1,3) = wy;
	A(2,0) = wy;	A(2,1) = -wx;	A(2,2) = 1;		A(2,3) = wz;
	A(3,0) = -wx;	A(3,1) = -wy;	A(3,2) = -wz;	A(3,3) = 1;

	return A;
}

dMatrix ARS (double gyro[3], double accl[3], double magn[3], double dt)
{
	GyroAverage (average_gyro, gyro, 10000, 100.);

	// ̷  ̾  ϱ  ӵ  ӵ հ .
	dVector w = dVector3 (
		_DEG2RAD*(gyro[0] - average_gyro[0]), 
		_DEG2RAD*(gyro[1] - average_gyro[1]), 
		_DEG2RAD*(gyro[2] - average_gyro[2]));

	/////////////////////////////////////////////////////////////////////////
	// Predict 

	// state transition matrix
	dMatrix A = GetStateTransitionMatrix (w[0], w[1], w[2], dt);
	
	const double _q[4][4] = { 
		{0.001, 0, 0, 0},
		{0, 0.001, 0, 0},
		{0, 0, 0.001, 0},
		{0, 0, 0, 0.001},
	};

	// Covariance matrix of porcess noises
	const dMatrix Q(4, 4, &_q[0][0]);

	// Predicted state estimate
	x = A*x;
	// Predicted estimate covariance
	P = A*P*~A + Q;

	/////////////////////////////////////////////////////////////////////////
	// Update 
	double phi_e   = atan2(-accl[1], -accl[2]);
	double theta_e = (-1 < accl[0] && accl[0] < 1) ? asin(-accl[0]/-1.) : 0.;

	// ߷°ӵ  1ó   ̵(k1) ū   Ѵ.
	double v = sqrt (accl[0]*accl[0] + accl[1]*accl[1] + accl[2]*accl[2]) - 1.;
	double s = exp(-v*v);
	double e = pow(2.*accl[0], 8);

	double _r[4][4] = { 
		{1/s+e, 0, 0, 0},
		{0, 1/s+e, 0, 0},
		{0, 0, 1/s+e, 0},
		{0, 0, 0, 1/s+e},
	};

	// Covariance matrix of observation noises
	dMatrix R(4, 4, &_r[0][0]);

	// measurement value
	Quaternion z(phi_e, theta_e, 0.);

	//  predicted measurement from the predicted state
	dMatrix h(4,1);
	h(0,0) = x(0,0);
	h(1,0) = x(1,0);
	h(2,0) = x(2,0);
	h(3,0) = x(3,0);

	// measurement residual
	// y = z - h
	dMatrix y(4,1);
	y(0,0) = z.v_[0] - h(0,0);
	y(1,0) = z.v_[1] - h(1,0);
	y(2,0) = z.v_[2] - h(2,0);
	y(3,0) = z.s_	 - h(3,0);

	const double _h[4][4] = { 
		{1, 0, 0, 0},
		{0, 1, 0, 0},
		{0, 0, 1, 0},
		{0, 0, 0, 1},
	};

	// observation matrix
	const dMatrix H(4, 4, &_h[0][0]);

	// Kalman gain
	dMatrix K = P*~H*!(H*P*~H + R);

	// Updated state estimate
	x = x + K*y;
	// Updated estimate covariance
	P = P - K*H*P;

	// ʹϾ ũ⸦ 1 .
	Quaternion q(x(3,0), x(0,0), x(1,0), x(2,0));
	q.unit();

	x(0,0) = q.v_[0];
	x(1,0) = q.v_[1];
	x(2,0) = q.v_[2];
	x(3,0) = q.s_;

	return q.RotationMatrix ();
}
